@@ -14,16 +14,16 @@ use noodles_sam::{
 
 use crate::record::sequence::Base;
 
-// § 4.2 The BAM format (2020-04-30)
+// § 4.2 The BAM format (2021-06-03)
 //
 // ref_id (4) + pos (4) + l_read_name (1) + mapq (1) + bin (2) + n_cigar_op (2) + flag (2) + l_seq
 // (4) + next_ref_id (4) + next_pos (4) + tlen (4)
-const BLOCK_HEADER_SIZE: usize = 32;
+const BLOCK_HEADER_SIZE: u32 = 32;
 
-// § 4.2.1 BIN field calculation (2020-04-30)
+// § 4.2.1 BIN field calculation (2021-06-03)
 const UNMAPPED_BIN: u16 = 4680;
 
-// § 4.2.3 SEQ and QUAL encoding (2020-04-30)
+// § 4.2.3 SEQ and QUAL encoding (2021-06-03)
 const NULL_QUALITY_SCORE: u8 = 255;
 
 pub fn write_sam_record<W>(
@@ -38,37 +38,37 @@ where
     let c_read_name =
         CString::new(name).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
 
-    let reference_sequence_id =
-        find_reference_sequence_id(reference_sequences, record.reference_sequence_name())?;
-    let mate_reference_sequence_id =
-        find_reference_sequence_id(reference_sequences, record.mate_reference_sequence_name())?;
-
     let read_name = c_read_name.as_bytes_with_nul();
     let l_read_name = u8::try_from(read_name.len())
         .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
     let n_cigar_op = u16::try_from(record.cigar().len())
         .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-    let l_seq = i32::try_from(record.sequence().len())
-        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-    let data_len = calculate_data_len(record.data())? as i32;
 
-    let block_size = BLOCK_HEADER_SIZE as i32
-        + i32::from(l_read_name)
-        + (4 * i32::from(n_cigar_op))
+    let l_seq = u32::try_from(record.sequence().len())
+        .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+
+    let data_len = calculate_data_len(record.data()).and_then(|len| {
+        u32::try_from(len).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))
+    })?;
+
+    let block_size = BLOCK_HEADER_SIZE
+        + u32::from(l_read_name)
+        + (4 * u32::from(n_cigar_op))
         + ((l_seq + 1) / 2)
         + l_seq
         + data_len;
+    writer.write_u32::<LittleEndian>(block_size)?;
 
-    writer.write_i32::<LittleEndian>(block_size)?;
+    // ref_id
+    write_reference_sequence_id(
+        writer,
+        reference_sequences,
+        record.reference_sequence_name(),
+    )?;
 
-    let ref_id = reference_sequence_id;
-    writer.write_i32::<LittleEndian>(ref_id)?;
-
-    let pos = record
-        .position()
-        .map(|v| i32::from(v) - 1)
-        .unwrap_or(crate::record::UNMAPPED_POSITION);
-    writer.write_i32::<LittleEndian>(pos)?;
+    // pos
+    write_position(writer, record.position())?;
 
     writer.write_u8(l_read_name)?;
 
@@ -85,7 +85,6 @@ where
             region_to_bin(start, end) as u16
         })
         .unwrap_or(UNMAPPED_BIN);
-
     writer.write_u16::<LittleEndian>(bin)?;
 
     writer.write_u16::<LittleEndian>(n_cigar_op)?;
@@ -93,16 +92,17 @@ where
     let flag = u16::from(record.flags());
     writer.write_u16::<LittleEndian>(flag)?;
 
-    writer.write_i32::<LittleEndian>(l_seq)?;
+    writer.write_u32::<LittleEndian>(l_seq)?;
 
-    let next_ref_id = mate_reference_sequence_id;
-    writer.write_i32::<LittleEndian>(next_ref_id)?;
+    // next_ref_id
+    write_reference_sequence_id(
+        writer,
+        reference_sequences,
+        record.mate_reference_sequence_name(),
+    )?;
 
-    let next_pos = record
-        .mate_position()
-        .map(|v| i32::from(v) - 1)
-        .unwrap_or(crate::record::UNMAPPED_POSITION);
-    writer.write_i32::<LittleEndian>(next_pos)?;
+    // next_pos
+    write_position(writer, record.mate_position())?;
 
     let tlen = record.template_length();
     writer.write_i32::<LittleEndian>(tlen)?;
@@ -111,7 +111,7 @@ where
 
     write_cigar(writer, record.cigar())?;
 
-    // § 4.2.3 SEQ and QUAL encoding (2020-04-30)
+    // § 4.2.3 SEQ and QUAL encoding (2021-06-03)
     let sequence = record.sequence();
     let quality_scores = record.quality_scores();
 
@@ -141,6 +141,47 @@ where
     Ok(())
 }
 
+fn write_reference_sequence_id<W>(
+    writer: &mut W,
+    reference_sequences: &ReferenceSequences,
+    reference_sequence_name: Option<&sam::record::ReferenceSequenceName>,
+) -> io::Result<()>
+where
+    W: Write,
+{
+    use crate::record::reference_sequence_id;
+
+    let id = match reference_sequence_name {
+        Some(name) => reference_sequences
+            .get_index_of(name.as_str())
+            .ok_or_else(|| {
+                io::Error::new(
+                    io::ErrorKind::InvalidInput,
+                    format!("invalid reference sequence name: {}", name),
+                )
+            })
+            .and_then(|i| {
+                i32::try_from(i).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))
+            })?,
+        None => reference_sequence_id::UNMAPPED,
+    };
+
+    writer.write_i32::<LittleEndian>(id)
+}
+
+fn write_position<W>(writer: &mut W, position: Option<sam::record::Position>) -> io::Result<()>
+where
+    W: Write,
+{
+    use crate::record::UNMAPPED_POSITION;
+
+    let pos = position
+        .map(|p| i32::from(p) - 1)
+        .unwrap_or(UNMAPPED_POSITION);
+
+    writer.write_i32::<LittleEndian>(pos)
+}
+
 fn write_cigar<W>(writer: &mut W, cigar: &Cigar) -> io::Result<()>
 where
     W: Write,
@@ -394,29 +435,7 @@ where
     }
 }
 
-fn find_reference_sequence_id(
-    reference_sequences: &ReferenceSequences,
-    reference_sequence_name: Option<&sam::record::ReferenceSequenceName>,
-) -> io::Result<i32> {
-    use crate::record::reference_sequence_id;
-
-    match reference_sequence_name {
-        Some(name) => reference_sequences
-            .get_index_of(name.as_str())
-            .ok_or_else(|| {
-                io::Error::new(
-                    io::ErrorKind::InvalidInput,
-                    format!("invalid reference sequence name: {}", name),
-                )
-            })
-            .and_then(|i| {
-                i32::try_from(i).map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))
-            }),
-        None => Ok(reference_sequence_id::UNMAPPED),
-    }
-}
-
-// § 5.3 C source code for computing bin number and overlapping bins (2020-04-30)
+// § 5.3 C source code for computing bin number and overlapping bins (2021-06-03)
 // 0-based, [start, end)
 #[allow(clippy::eq_op)]
 pub(crate) fn region_to_bin(start: i32, mut end: i32) -> i32 {
@@ -532,34 +551,55 @@ mod tests {
     }
 
     #[test]
-    fn test_find_reference_sequence_id() -> Result<(), Box<dyn std::error::Error>> {
+    fn test_write_reference_sequence_id() -> Result<(), Box<dyn std::error::Error>> {
         use sam::header::ReferenceSequence;
 
-        let reference_sequences = vec![("sq0", 8), ("sq1", 13), ("sq2", 21)]
+        let reference_sequences = vec![("sq0", 8), ("sq1", 13)]
             .into_iter()
             .map(|(name, len)| ReferenceSequence::new(name, len).map(|rs| (name.into(), rs)))
             .collect::<Result<_, _>>()?;
 
-        assert!(matches!(
-            find_reference_sequence_id(&reference_sequences, None),
-            Ok(-1)
-        ));
+        let mut buf = Vec::new();
 
-        let name = "sq1".parse().map(Some)?;
-        assert!(matches!(
-            find_reference_sequence_id(&reference_sequences, name.as_ref()),
-            Ok(1)
-        ));
+        buf.clear();
+        let reference_sequence_name = "sq0".parse()?;
+        write_reference_sequence_id(
+            &mut buf,
+            &reference_sequences,
+            Some(&reference_sequence_name),
+        )?;
+        assert_eq!(buf, [0x00, 0x00, 0x00, 0x00]);
 
-        let name = "sq3".parse().map(Some)?;
+        buf.clear();
+        write_reference_sequence_id(&mut buf, &reference_sequences, None)?;
+        assert_eq!(buf, [0xff, 0xff, 0xff, 0xff]);
+
+        buf.clear();
+        let reference_sequence_name = "sq2".parse()?;
         assert!(matches!(
-            find_reference_sequence_id(&reference_sequences, name.as_ref()),
+            write_reference_sequence_id(&mut buf, &reference_sequences, Some(&reference_sequence_name)),
             Err(ref e) if e.kind() == io::ErrorKind::InvalidInput,
         ));
 
         Ok(())
     }
 
+    #[test]
+    fn test_write_position() -> Result<(), Box<dyn std::error::Error>> {
+        let mut buf = Vec::new();
+
+        buf.clear();
+        let position = sam::record::Position::try_from(8)?;
+        write_position(&mut buf, Some(position))?;
+        assert_eq!(buf, [0x07, 0x00, 0x00, 0x00]); // pos = 7
+
+        buf.clear();
+        write_position(&mut buf, None)?;
+        assert_eq!(buf, [0xff, 0xff, 0xff, 0xff]); // pos = -1
+
+        Ok(())
+    }
+
     #[test]
     fn test_region_to_bin() {
         // § 5.3 C source code for computing bin number and overlapping bins (2021-01-07)
